/*
*  RAL -- Rubrica Addressbook Library
*  file: gmail_csv.c
* 
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <glib-object.h>
#include <glib/gi18n-lib.h>

#include "gmail_csv.h"

#include "abook.h"
#include "personal.h"
#include "company.h"
#include "contact.h"
#include "net.h"
#include "notes.h"
#include "plugin.h"
#include "error.h"


#define BUFSIZE 256

#define COMMA_DELIM ','
#define TAB_DELIM   '\t'
#define QUOTES      '"'


#define R_GMAIL_CSV_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o),      \
			            R_GMAIL_CSV_TYPE, RGMailCsvPrivate))


/* GMail csv fields (coded as utf-16):
   Name, Email, Notes
 */
typedef enum {
  R_GMAIL_CSV_NAME = 0,          
  R_GMAIL_CSV_EMAIL,         
  R_GMAIL_CSV_NOTES,     
  R_GMAIL_CSV_BUILD_CARD
} RGMailCsvField;


// static GObject* parent_class = NULL;


struct _RGMailCsvPrivate{
  glong pos;
  glong lines;
  
  gchar* content;            /* file content converted to utf8 */
  gchar* buffer[3];
  
  gboolean dispose_has_run;
};



static void   r_gmail_csv_init       (RGMailCsv* obj);
static void   r_gmail_csv_class_init (RGMailCsvClass* klass);

static void   r_gmail_csv_dispose    (GObject* obj);
static void   r_gmail_csv_finalize   (GObject* obj);

static gchar _get_char       (RGMailCsv* csv);
static gchar _peek_next_char (RGMailCsv* csv);


GType
r_gmail_csv_get_type()
{
  static GType r_gmail_csv_type = 0;
  
  if (!r_gmail_csv_type)
    {
      static const GTypeInfo r_gmail_csv_info =
	{
	  sizeof(RGMailCsvClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) r_gmail_csv_class_init,
	  NULL,
	  NULL,
	  sizeof(RGMailCsv),
	  0,
	  (GInstanceInitFunc) r_gmail_csv_init
	};

      r_gmail_csv_type = g_type_register_static (G_TYPE_OBJECT, "RGMailCsv",
						 &r_gmail_csv_info, 0);
    }
  
  return r_gmail_csv_type;
}


static void
r_gmail_csv_class_init(RGMailCsvClass* klass)
{
  GObjectClass *class;
  
  class = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) r_gmail_csv_dispose;
  class->finalize = (GObjectFinalizeFunc) r_gmail_csv_finalize;

  g_type_class_add_private (klass, sizeof(RGMailCsvPrivate));
}


static void
r_gmail_csv_init(RGMailCsv* self)
{
  g_return_if_fail(IS_R_GMAIL_CSV(self));
  
  self->priv = R_GMAIL_CSV_GET_PRIVATE(self);

  self->priv->pos = 0;
  self->priv->lines = 0;
  self->priv->content  = NULL;
  self->priv->buffer[R_GMAIL_CSV_NAME]  = NULL;
  self->priv->buffer[R_GMAIL_CSV_EMAIL] = NULL;
  self->priv->buffer[R_GMAIL_CSV_NOTES] = NULL;

  self->priv->dispose_has_run = FALSE;
}

static void 
r_gmail_csv_dispose (GObject* obj)
{
  RGMailCsv* self = R_GMAIL_CSV(obj);
  RGMailCsvPrivate* priv;

  g_return_if_fail(IS_R_GMAIL_CSV(obj));
  
  priv = R_GMAIL_CSV_GET_PRIVATE(self);

  if (priv->dispose_has_run)
    return;

  g_free(priv->buffer);
  priv->dispose_has_run = TRUE;
}


static void 
r_gmail_csv_finalize (GObject* obj)
{
  RGMailCsv* self = R_GMAIL_CSV(obj);

  g_return_if_fail(IS_R_GMAIL_CSV(self));
}


static gboolean 
r_gmail_csv_is_separator(gunichar ch)
{
  if ((ch == COMMA_DELIM) ||   /* comma separator */
      (ch == TAB_DELIM))       /* tab   separator */
    return TRUE;
  
  return FALSE;
}


static gchar 
_get_char (RGMailCsv* csv)
{
  RGMailCsvPrivate* priv;
  gchar ret;

  g_return_val_if_fail(IS_R_GMAIL_CSV(csv), '\0');

  priv  = R_GMAIL_CSV_GET_PRIVATE(csv);
  ret = priv->content[priv->pos++];
  
  return ret;
}

static gchar 
_peek_next_char (RGMailCsv* csv)
{
  RGMailCsvPrivate* priv;
  gchar ret;

  g_return_val_if_fail(IS_R_GMAIL_CSV(csv), '\0');

  priv = R_GMAIL_CSV_GET_PRIVATE(csv);

  ret = priv->content[priv->pos];

  return ret;
}




/* *****************************  Public functions  ***********************
 */

/**
 * r_gmail_csv_new
 *
 * create a new #RGMailCsv
 *
 * returns: a #RGMailCsv*
 */
RGMailCsv* 
r_gmail_csv_new(void)
{
  return (RGMailCsv*) g_object_new(r_gmail_csv_get_type(), NULL);
}


/**
 * r_gmail_csv_free
 * @csv: a #RGMailCsv
 * 
 * free the #RGMailCsv object
 */
void  
r_gmail_csv_free(RGMailCsv *csv) 
{ 
  g_return_if_fail(IS_R_GMAIL_CSV(csv)); 

  g_object_unref(csv); 
} 


/**
 * r_gmail_csv_open_file
 * @csv: #RGMailCsv
 * @file: the file's name
 *
 * Read a csv file. 
 * When a csv file is open a "file_opened" signal is emitted, 
 * with an error code on file status.
 * Each time a csv's record is decoded, a "card_decoded" signal is 
 * emitted and a #RPersonalCard will be sent to the user callback
 * function. The "addressbook_decoded" signal is emitted when csv file
 * if successfully read.
 *
 * returns: a gint, %FILE_OPEN_FAIL or %FILE_IMPORT_SUCCESS
 */
static gboolean          
r_gmail_csv_open_file (RAbook* abook, const gchar* filename)
{
  RGMailCsv* csv;
  RGMailCsvPrivate* priv;
  gunichar2* file_contents;
  GError* error = NULL;
  gint i = 0;
  gchar ch;
  gchar name[BUFSIZE];
  gchar email[BUFSIZE];
  gchar notes[BUFSIZE];  
  gint state;

  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);

  memset(&name, 0, BUFSIZE);
  memset(&email, 0, BUFSIZE);
  memset(&notes, 0, BUFSIZE);


  csv = (RGMailCsv*) r_abook_get_plugin(abook);

  if (!IS_R_GMAIL_CSV(csv))
    {
      g_signal_emit_by_name(R_ABOOK(abook), "open_fail", 
			    PLUGIN_ERROR, G_TYPE_INT);
      
      return FALSE;
    }
  
  if (!filename)
    {
      g_signal_emit_by_name(R_ABOOK(abook), "open_fail", 
			    NO_FILENAME, G_TYPE_INT);
      
      return FALSE;
    }
  
  if (!g_file_test(filename, G_FILE_TEST_EXISTS))
    {
      g_signal_emit_by_name(R_ABOOK(abook), "open_fail", 
			    FILE_NOT_EXIST, G_TYPE_INT);
      
      return FALSE;
    }
  
  if (!g_file_get_contents(filename, &file_contents, NULL, &error))
    {
      g_warning("\n%s", error->message);

      g_signal_emit_by_name(R_ABOOK(abook), "open_fail", 
			    FILE_OPEN_FAIL, G_TYPE_INT);

      return FALSE; 
    }

  error = NULL;
  priv  = R_GMAIL_CSV_GET_PRIVATE(csv);
  priv->content = g_utf16_to_utf8((gunichar2*) file_contents, 
				  -1, NULL, NULL, &error);
  if (!priv->content)
    {
      g_warning("\n%s", error->message);
      
      g_signal_emit_by_name(R_ABOOK(abook), "open_fail", 
			    FILE_OPEN_FAIL, G_TYPE_INT);
      
      return FALSE; 
    }    
  
  g_free(file_contents);
  
  g_strstrip(priv->content);
  g_object_set(R_ABOOK(abook),
	       "addressbook-name", g_path_get_basename(filename),
	       "addressbook-path", g_path_get_dirname(filename),
	       NULL);

  state = R_GMAIL_CSV_NAME;
  name[0] = email[0] = notes[0] = '\0';
  do 
    {
      ch = _get_char(csv);
      
      switch (state)
	{
	case R_GMAIL_CSV_NAME:
	  if (((ch == '\r') && (_peek_next_char(csv) == '\n')))
	    {
	      name[i] = '\0';
	      state = R_GMAIL_CSV_BUILD_CARD;
	      _get_char(csv);
	      i = 0;
	      
	      break;
	    }
	  
	  if (r_gmail_csv_is_separator(ch))
	    {
	      name[i] = '\0';
	      state = R_GMAIL_CSV_EMAIL;
	      i = 0;
	    }
	  else
	    name[i++] = ch;
	  break;

	case R_GMAIL_CSV_EMAIL:
	  if (((ch == '\r') && (_peek_next_char(csv) == '\n')))
	    {
	      email[i] = '\0';
	      state = R_GMAIL_CSV_BUILD_CARD;
	      _get_char(csv);
	      i = 0;
	      
	      break;
	    }
	  
	  if (r_gmail_csv_is_separator(ch))
	    {
	      email[i] = '\0';
	      state = R_GMAIL_CSV_NOTES;
	      i = 0;
	    }
	  else
	    email[i++] = ch;
	  break;

	case R_GMAIL_CSV_NOTES:
	  if (((ch == '\r') && (_peek_next_char(csv) == '\n')))
	    {
	      notes[i] = '\0';
	      state = R_GMAIL_CSV_BUILD_CARD;
	      _get_char(csv);
	      i = 0;

	      break;
	    }

	  if (r_gmail_csv_is_separator(ch))
	    {
	      notes[i] = '\0';
	      state = R_GMAIL_CSV_BUILD_CARD;
	      i = 0;
	    }
	  else
	    notes[i++] = ch;
	  break;

	case R_GMAIL_CSV_BUILD_CARD:
	  priv->buffer[R_GMAIL_CSV_NAME]  = name;
	  priv->buffer[R_GMAIL_CSV_EMAIL] = email;
	  priv->buffer[R_GMAIL_CSV_NOTES] = notes;
	  
	  g_signal_emit_by_name(R_ABOOK(abook), "card_read", 
				priv->buffer, G_TYPE_POINTER);

	  state = R_GMAIL_CSV_NAME;
	  memset(&name, 0, BUFSIZE);
	  memset(&email, 0, BUFSIZE);
	  memset(&notes, 0, BUFSIZE);

	  i = 0;
	  priv->lines++;
	  break;

	default:
	  break;
	}
    }
  while(ch);

  g_signal_emit_by_name(R_ABOOK(abook), "addressbook_read", NULL, G_TYPE_NONE);
  
  return TRUE;
}


static void
write_card (RCard* card, FILE* fp)
{
  RNotes*      notes;
  RNetAddress* net;

  gchar *type;
  gchar *name  = NULL;
  gchar *email = NULL;
  gchar *note  = NULL;

  g_object_get(card, "card-type", &type, NULL);
  if (g_ascii_strcasecmp(type, "personal") == 0)
    {
      RContact* contact;
  
      contact = r_personal_card_get_contact(R_PERSONAL_CARD(card));
      notes   = r_personal_card_get_notes(R_PERSONAL_CARD(card));
      
      g_object_get(contact, "first-name", &name, NULL);
      g_object_get(notes, "other-notes", &note, NULL);      
    }
  else // company
    {
      g_object_get(R_COMPANY_CARD(card), "notes", &note, NULL);           
    }

  /* reset net iter */
  r_card_reset_net_address(card);
  
  /* find first net address */
  net = r_card_find_net_address(card, R_NET_ADDRESS_EMAIL);
  if (net) 
    g_object_get(net, "url", &email, NULL); 
  
  fprintf(fp, "\"%s\",\"%s\",\"%s\"\n", 
	  name ? name : "",  email ? email : "", note ? note : "");  
}


static gboolean 
r_gmail_csv_save_file (RAbook* abook, gchar* filename)
{  
  FILE* fp;
  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);
  g_return_val_if_fail(filename != NULL, FALSE);

  if (!(fp = fopen(filename, "w")))
    {
      g_warning("\nCan't write file: %s", filename);
      
      return FALSE;
    }
  
  r_abook_foreach_card (abook, (RFunc) write_card, fp);
  fflush(fp);

  return TRUE;
}


static gboolean
r_gmail_csv_overwrite_file(RAbook* abook)
{
  gchar* file;
  gchar* path;
  gchar* filename;
  
  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);
  
  g_object_get(abook, "addressbook-path", &path, 
	       "addressbook-name", &file, NULL);
  filename = g_strdup_printf("%s%s%s", path, G_DIR_SEPARATOR_S, file);
  
  if (g_file_test(filename, G_FILE_TEST_EXISTS))
    g_remove(filename);

  if (!r_gmail_csv_save_file(abook, filename))
    {
      g_signal_emit_by_name(abook, "save_fail", OVERWRITING, G_TYPE_INT);
      
      g_free(filename);
      return FALSE;
    }
    
  g_free(filename);
  g_signal_emit_by_name(abook, "addressbook_saved", NULL, G_TYPE_NONE);  
  
  return TRUE;  
}


/**
 * r_gmail_csv_read_previous_record
 * @csv: #RGMailCsv
 *
 * read the previous record from csv file
 */
void        
r_gmail_csv_read_previous_record (RGMailCsv *csv) 
{
  g_return_if_fail(IS_R_GMAIL_CSV(csv)); 
}





G_MODULE_EXPORT void 
plugin_init (RPlugin* plugin, gchar* file)
{
  RPluginAction* action;
  RFilter* filter;

  g_return_if_fail(plugin != NULL);
 
#ifdef ENABLE_DEBUG
  g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "plugin_init");
  g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Initializing csvgmail plugin");
#else
  g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "Initializing csvgmail plugin");
#endif
  g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "Don't use the csvgmail plugin, it is in development stage and isn't ready");


  r_plugin_set_obj(plugin, r_gmail_csv_new());
  g_object_set(plugin, 
	       "plugin-name", "gmail_csv", 
	       "plugin-filename", file,
	       "plugin-info", "This plugin manages the gmail csv file format",
	       "plugin-configurable", TRUE, NULL);
  
  filter = r_filter_new();
  g_object_set(filter, 
	       "filter-name", "GMAIL CSV", 
	       "filter-mime", "text/x-comma-separated-values", NULL);

  action = g_malloc(sizeof(RPluginAction));
  action->name   = g_strdup("read");
  action->handle = (gpointer) r_gmail_csv_open_file;
  r_plugin_add_action(plugin, action);  
    
  action = g_malloc(sizeof(RPluginAction));
  action->name   = g_strdup("write");
  action->handle = (gpointer) r_gmail_csv_save_file;
  r_plugin_add_action(plugin, action);  

  action = g_malloc(sizeof(RPluginAction));
  action->name   = g_strdup("overwrite");
  action->handle = (gpointer) r_gmail_csv_overwrite_file;
  r_plugin_add_action(plugin, action);  
}


G_MODULE_EXPORT void     
plugin_fini (void)
{
  // TODO: liberare la memoria e rilasciare l'handler al plugin
}
